﻿using ContextAop.Attribute;
using ContextAop.Enum;
using System;
using System.Reflection;
using System.Runtime.Remoting.Activation;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Proxies;

namespace ContextAop.Proxy
{
    public class AopRealProxy : RealProxy
    {
        private MarshalByRefObject _target;

        public AopRealProxy(Type type, MarshalByRefObject target) : base(type)
        {
            this._target = target;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="msg"></param>
        /// <returns></returns>
        public override IMessage Invoke(IMessage msg)
        {
            IMethodCallMessage methodCall = (IMethodCallMessage)msg;
            IConstructionCallMessage ctr = methodCall as IConstructionCallMessage;
            IMessage message = null;

            InterceptionType interceptionType = GetInterceptionType(methodCall);

            //构造函数
            if (ctr != null)
            {
                IConstructionReturnMessage constructionReturnMessage = this.InitializeServerObject((IConstructionCallMessage)msg);
                RealProxy.SetStubData(this, constructionReturnMessage.ReturnValue);
                message = constructionReturnMessage;
            }
            //方法
            else
            {
                if (interceptionType == InterceptionType.Before || interceptionType == InterceptionType.None)
                    Console.WriteLine("开始执行");
                try
                {
                    var methodInfo = methodCall.MethodBase as MethodInfo;
                    var result = methodInfo.Invoke(GetUnwrappedServer(), methodCall.InArgs);
                    message= new ReturnMessage(result, null, 0, methodCall.LogicalCallContext, methodCall);
                    if (interceptionType == InterceptionType.After || interceptionType == InterceptionType.None)
                        Console.WriteLine("结束执行");
                }
                catch (Exception ex)
                {
                    message = new ReturnMessage(ex, methodCall);
                    if (interceptionType == InterceptionType.Exception || interceptionType == InterceptionType.None)
                        Console.WriteLine("异常执行");
                }
            }
            return message;
        }

        /// <summary>
        /// 获取方法标签类型
        /// </summary>
        private InterceptionType GetInterceptionType(IMethodCallMessage MethodCall)
        {
            InterceptionType type = InterceptionType.None;
            foreach (System.Attribute attr in MethodCall.MethodBase.GetCustomAttributes(false))
            {
                MethodInterceptionAttribute methodInterceptionAttr = attr as MethodInterceptionAttribute;
                if (null !=methodInterceptionAttr)
                {
                    type = methodInterceptionAttr.AdviceType;
                    break;
                }
            }
            return type;
        }

    }
}
