﻿using ContextAop.Attribute;
using ContextAop.Enum;
using ContextAop.Transaction;
using System;
using System.Reflection;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Activation;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Proxies;

namespace ContextAop.Proxy
{
    public class TransactionAopRealProxy : RealProxy
    {
        private MarshalByRefObject _target;

        private ITransaction transactionService = null;
        public TransactionAopRealProxy(Type type, MarshalByRefObject target) : base(type)
        {
            this._target = target;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="msg"></param>
        /// <returns></returns>
        public override IMessage Invoke(IMessage msg)
        {
            IMethodCallMessage methodCall = (IMethodCallMessage)msg;
            IConstructionCallMessage ctr = methodCall as IConstructionCallMessage;
            IMessage message = null;

            //构造函数
            if (ctr != null)
            {
                IConstructionReturnMessage constructionReturnMessage = this.InitializeServerObject((IConstructionCallMessage)msg);
                RealProxy.SetStubData(this, constructionReturnMessage.ReturnValue);
                message = constructionReturnMessage;
            }
            //方法
            else
            {
                transactionService.BeginTransaction();
                try
                {
                    var methodInfo = methodCall.MethodBase as MethodInfo;
                    var result = methodInfo.Invoke(GetUnwrappedServer(), methodCall.InArgs);
                    message = new ReturnMessage(result, null, 0, methodCall.LogicalCallContext, methodCall);
                    transactionService.CommitTransaction();
                }
                catch (Exception ex)
                {
                    transactionService.RollbackTransaction();
                    message = new ReturnMessage(ex, methodCall);
                }
            }
            return message;
        }


        /// <summary>
        /// 事务入侵
        /// </summary>
        /// <param name="transactionService"></param>
        public void InterceptionTransaction(ITransaction transactionService)
        {
            this.transactionService = transactionService;
        }
    }
}
